@@ -1354,23 +1354,8 @@ main_dialog (GimpDrawable *drawable)
   GtkWidget *frame;
   GtkWidget *button;
   GtkWidget *toggle;
-  gchar **argv;
-  gint    argc;
 
-  argc    = 1;
-  argv    = g_new (gchar *, 1);
-  argv[0] = g_strdup ("MapObject");
-
-  gtk_init (&argc, &argv);
-  gtk_rc_parse (gimp_gtkrc ());
-
-  if (! gimp_use_xshm ())
-    gdk_set_use_xshm (FALSE);
-
-  /* Set up ArcBall stuff */
-  /* ==================== */
-
-  /*ArcBall_Init(); */
+  gimp_ui_init ("MapObject", FALSE);
 
   visinfo = gck_visualinfo_new ();
 
