@@ -120,11 +120,11 @@
  */
 
 
-#include "config.h"		/* configure cares about HAVE_PROGRESSIVE_JPEG */
+#include "config.h"   /* configure cares about HAVE_PROGRESSIVE_JPEG */
 
-#include <glib.h>		/* We want glib.h first because of some
-				 * pretty obscure Win32 compilation issues.
-				 */
+#include <glib.h>     /* We want glib.h first because of some
+		       * pretty obscure Win32 compilation issues.
+		       */
 #include <setjmp.h>
 #include <signal.h>
 #include <stdio.h>
@@ -164,13 +164,13 @@
 #define DEFAULT_COMMENT     "Created with The GIMP"
 
 /* sg - these should not be global... */
-static gint32 volatile  image_ID_global = -1;
-static gint32           orig_image_ID_global;
-static gint32           drawable_ID_global = -1;
-static gint32           layer_ID_global = -1;
-static GtkWidget       *preview_size = NULL;
-static GimpDrawable       *drawable_global = NULL;
-
+static gint32 volatile  image_ID_global       = -1;
+static gint32           orig_image_ID_global  = -1;
+static gint32           drawable_ID_global    = -1;
+static gint32           layer_ID_global       = -1;
+static GtkWidget       *preview_size          = NULL;
+static GimpDrawable    *drawable_global       = NULL;
+ 
 typedef struct
 {
   gdouble quality;
@@ -192,61 +192,56 @@ typedef struct
 typedef struct 
 {
   struct jpeg_compress_struct cinfo;
-  gint       tile_height;
-  FILE      *outfile;
-  gint       has_alpha;
-  gint       rowstride;
-  guchar    *temp;
-  guchar    *data;
-  guchar    *src;
+  gint          tile_height;
+  FILE         *outfile;
+  gboolean      has_alpha;
+  gint          rowstride;
+  guchar       *temp;
+  guchar       *data;
+  guchar       *src;
   GimpDrawable *drawable;
   GimpPixelRgn  pixel_rgn;
-  gchar     *file_name;
-  gint       abort_me;
-} preview_persistent;
+  gchar        *file_name;
+  gboolean      abort_me;
+} PreviewPersistent;
+
+static gboolean *abort_me = NULL;
 
-gint *abort_me = NULL;
 
 /* Declare local functions.
  */
-static void   query                     (void);
-static void   run                       (gchar         *name,
-					 gint           nparams,
-					 GimpParam        *param,
-					 gint          *nreturn_vals,
-					 GimpParam       **return_vals);
-static gint32 load_image                (gchar         *filename, 
-					 GimpRunModeType   runmode, 
-					 gint           preview);
-static gint   save_image                (gchar         *filename,
-					 gint32         image_ID,
-					 gint32         drawable_ID,
-					 gint32         orig_image_ID,
-					 gint           preview);
+static void     query                     (void);
+static void     run                       (gchar            *name,
+					   gint              nparams,
+					   GimpParam        *param,
+					   gint             *nreturn_vals,
+					   GimpParam       **return_vals);
+static gint32   load_image                (gchar            *filename, 
+					   GimpRunModeType   runmode, 
+					   gboolean          preview);
+static gboolean save_image                (gchar            *filename,
+					   gint32            image_ID,
+					   gint32            drawable_ID,
+					   gint32            orig_image_ID,
+					   gboolean          preview);
 
-static void   add_menu_item             (GtkWidget *menu,
-					 char *label,
-					 guint op_no,
-					 GtkSignalFunc callback);
+static gboolean save_dialog                (void);
 
-static gint   save_dialog                (void);
+static void     save_close_callback        (GtkWidget       *widget,
+					    gpointer         data);
+static void     save_ok_callback           (GtkWidget       *widget,
+					    gpointer         data);
+static void     save_restart_toggle_update (GtkWidget       *toggle,
+					    GtkAdjustment   *adjustment);
+static void     save_restart_update        (GtkAdjustment   *adjustment,
+					    GtkWidget       *toggle);
 
-static void   save_close_callback        (GtkWidget *widget,
-					  gpointer   data);
-static void   save_ok_callback           (GtkWidget *widget,
-					  gpointer   data);
-static void   save_restart_toggle_update (GtkWidget     *toggle,
-					  GtkAdjustment *adjustment);
-static void   save_restart_update        (GtkAdjustment *adjustment,
-					  GtkWidget     *toggle);
+static void     make_preview               (void);
+static void     destroy_preview            (void);
 
-static void   make_preview               (void);
-static void   destroy_preview            (void);
+static void     menu_callback              (GtkWidget       *widget, 
+					    gpointer         data);
 
-static void   subsmp_callback            (GtkWidget *widget, 
-					  gpointer data);
-static void   dct_callback               (GtkWidget *widget, 
-					  gpointer data);
 
 GimpPlugInInfo PLUG_IN_INFO =
 {
@@ -274,11 +269,12 @@ static JpegSaveInterface jsint =
   FALSE   /*  run  */
 };
 
-static gchar     *image_comment = NULL;
 
+static gchar     *image_comment         = NULL;
 static GtkWidget *restart_markers_scale = NULL;
 static GtkWidget *restart_markers_label = NULL;
 
+
 MAIN ()
 
 static void
@@ -351,24 +347,24 @@ query (void)
 }
 
 static void
-run (gchar   *name,
-     gint     nparams,
+run (gchar      *name,
+     gint        nparams,
      GimpParam  *param,
-     gint    *nreturn_vals,
+     gint       *nreturn_vals,
      GimpParam **return_vals)
 {
-  static GimpParam values[2];
-  GimpRunModeType  run_mode;
-  GimpPDBStatusType   status = GIMP_PDB_SUCCESS;
-  gint32        image_ID;
-  gint32        drawable_ID;
-  gint32        orig_image_ID;
-  gint32        display_ID = -1;
+  static GimpParam      values[2];
+  GimpRunModeType       run_mode;
+  GimpPDBStatusType     status = GIMP_PDB_SUCCESS;
+  gint32                image_ID;
+  gint32                drawable_ID;
+  gint32                orig_image_ID;
+  gint32                display_ID = -1;
 #ifdef GIMP_HAVE_PARASITES
-  GimpParasite *parasite;
+  GimpParasite         *parasite;
 #endif /* GIMP_HAVE_PARASITES */
-  gint          err;
-  GimpExportReturnType export = GIMP_EXPORT_CANCEL;
+  gboolean              err;
+  GimpExportReturnType  export = GIMP_EXPORT_CANCEL;
 
   run_mode = param[0].data.d_int32;
 
@@ -379,7 +375,7 @@ run (gchar   *name,
 
   if (strcmp (name, "file_jpeg_load") == 0)
     {
-      INIT_I18N();
+      INIT_I18N_UI();
 
       image_ID = load_image (param[1].data.d_string, run_mode, FALSE);
 
@@ -428,12 +424,10 @@ run (gchar   *name,
 	default:
 	  break;
 	}
-
-      if (image_comment) 
-	{
-	  g_free (image_comment);
-	  image_comment = NULL;
-	}
+      
+      g_free (image_comment);
+      image_comment = NULL;
+    
 #ifdef GIMP_HAVE_PARASITES
       parasite = gimp_image_parasite_find (orig_image_ID, "gimp-comment");
       if (parasite) 
@@ -525,8 +519,7 @@ run (gchar   *name,
 	      jsvals.preview     = FALSE;
 
 	      /* free up the default -- wasted some effort earlier */
-	      if (image_comment) 
-		g_free (image_comment);
+	      g_free (image_comment);
 	      image_comment = g_strdup (param[9].data.d_string);
 
 	      if (jsvals.quality < 0.0 || jsvals.quality > 1.0)
@@ -600,7 +593,7 @@ run (gchar   *name,
        * and add new ones. */
       
       gimp_image_parasite_detach (orig_image_ID, "gimp-comment");
-      if (strlen (image_comment)) 
+      if (image_comment && strlen (image_comment)) 
 	{
 	  parasite = gimp_parasite_new ("gimp-comment",
 					GIMP_PARASITE_PERSISTENT,
@@ -629,7 +622,7 @@ run (gchar   *name,
 static guint
 jpeg_getc (j_decompress_ptr cinfo)
 {
-  struct jpeg_source_mgr * datasrc = cinfo->src;
+  struct jpeg_source_mgr *datasrc = cinfo->src;
 
   if (datasrc->bytes_in_buffer == 0) 
     {
@@ -637,6 +630,7 @@ jpeg_getc (j_decompress_ptr cinfo)
 	ERREXIT (cinfo, JERR_CANT_SUSPEND);
     }
   datasrc->bytes_in_buffer--;
+
   return *datasrc->next_input_byte++;
 }
 
@@ -654,8 +648,8 @@ static GString *local_image_comments = NULL;
 static boolean
 COM_handler (j_decompress_ptr cinfo)
 {
-  int length;
-  unsigned int ch;
+  gint  length;
+  guint ch;
 
   length = jpeg_getc (cinfo) << 8;
   length += jpeg_getc (cinfo);
@@ -703,34 +697,33 @@ my_error_exit (j_common_ptr cinfo)
 }
 
 static gint32
-load_image (gchar        *filename, 
+load_image (gchar           *filename, 
 	    GimpRunModeType  runmode, 
-	    gint          preview)
+	    gboolean         preview)
 {
-  GimpPixelRgn pixel_rgn;
-  GimpDrawable *drawable;
-  gint32 volatile image_ID;
-  gint32 layer_ID;
+  GimpPixelRgn     pixel_rgn;
+  GimpDrawable    *drawable;
+  gint32 volatile  image_ID;
+  gint32           layer_ID;
   struct jpeg_decompress_struct cinfo;
-  struct my_error_mgr jerr;
-  FILE *infile;
-  guchar *buf;
-  guchar * volatile padded_buf = NULL;
+  struct my_error_mgr           jerr;
+  FILE    *infile;
+  guchar  *buf;
+  guchar  * volatile padded_buf = NULL;
   guchar **rowbuf;
-  gchar *name;
-  gint image_type;
-  gint layer_type;
-  gint tile_height;
-  gint scanlines;
-  gint i, start, end;
+  gchar   *name;
+  gint     image_type;
+  gint     layer_type;
+  gint     tile_height;
+  gint     scanlines;
+  gint     i, start, end;
 
 #ifdef GIMP_HAVE_PARASITES
   JpegSaveVals local_save_vals;
   GimpParasite * volatile comment_parasite = NULL;
-  GimpParasite * volatile vals_parasite = NULL;
+  GimpParasite * volatile vals_parasite    = NULL;
 #endif /* GIMP_HAVE_PARASITES */
 
-  
   /* We set up the normal JPEG error routines. */
   cinfo.err = jpeg_std_error (&jerr.pub);
   jerr.pub.error_exit = my_error_exit;
@@ -785,46 +778,47 @@ load_image (gchar        *filename,
    */
 
 #ifdef GIMP_HAVE_PARASITES
-  if (!preview) {
-    /* if we had any comments then make a parasite for them */
-    if (local_image_comments) 
-      {
-	char *string = local_image_comments->str;
-	g_string_free (local_image_comments,FALSE);
-	local_image_comments = NULL;
-	comment_parasite = gimp_parasite_new ("gimp-comment",
-					      GIMP_PARASITE_PERSISTENT,
-					      strlen (string) + 1, string);
-      } 
-    else 
-      {
-	comment_parasite = NULL;
-      }
-  
-    /* pw - figuring out what the saved values were is non-trivial.
-     * They don't seem to be in the cinfo structure. For now, I will
-     * just use the defaults, but if someone figures out how to derive
-     * them this is the place to store them. */
-
-    local_save_vals.quality     = DEFAULT_QUALITY;
-    local_save_vals.smoothing   = DEFAULT_SMOOTHING;
-    local_save_vals.optimize    = DEFAULT_OPTIMIZE;
-  
+  if (!preview) 
+    {
+      /* if we had any comments then make a parasite for them */
+      if (local_image_comments && local_image_comments->len) 
+	{
+	  gchar *string = local_image_comments->str;
+	  g_string_free (local_image_comments, FALSE);
+	  local_image_comments = NULL;
+	  comment_parasite = gimp_parasite_new ("gimp-comment",
+						GIMP_PARASITE_PERSISTENT,
+						strlen (string) + 1, string);
+	} 
+      else 
+	{
+	  comment_parasite = NULL;
+	}
+      
+      /* pw - figuring out what the saved values were is non-trivial.
+       * They don't seem to be in the cinfo structure. For now, I will
+       * just use the defaults, but if someone figures out how to derive
+       * them this is the place to store them. */
+      
+      local_save_vals.quality     = DEFAULT_QUALITY;
+      local_save_vals.smoothing   = DEFAULT_SMOOTHING;
+      local_save_vals.optimize    = DEFAULT_OPTIMIZE;
+      
 #ifdef HAVE_PROGRESSIVE_JPEG 
-    local_save_vals.progressive = cinfo.progressive_mode;
+      local_save_vals.progressive = cinfo.progressive_mode;
 #else
-    local_save_vals.progressive = 0;
+      local_save_vals.progressive = 0;
 #endif /* HAVE_PROGRESSIVE_JPEG */
-    local_save_vals.baseline    = DEFAULT_BASELINE;
-    local_save_vals.subsmp      = DEFAULT_SUBSMP;   /* sg - this _is_ there, but I'm too lazy */ 
-    local_save_vals.restart     = DEFAULT_RESTART;
-    local_save_vals.dct         = DEFAULT_DCT;
-    local_save_vals.preview     = DEFAULT_PREVIEW;
-  
-    vals_parasite = gimp_parasite_new ("jpeg-save-options", 0,
-				       sizeof (local_save_vals),
-				       &local_save_vals);
-  } 
+      local_save_vals.baseline    = DEFAULT_BASELINE;
+      local_save_vals.subsmp      = DEFAULT_SUBSMP;   /* sg - this _is_ there, but I'm too lazy */ 
+      local_save_vals.restart     = DEFAULT_RESTART;
+      local_save_vals.dct         = DEFAULT_DCT;
+      local_save_vals.preview     = DEFAULT_PREVIEW;
+      
+      vals_parasite = gimp_parasite_new ("jpeg-save-options", 0,
+					 sizeof (local_save_vals),
+					 &local_save_vals);
+    } 
 #endif /* GIMP_HAVE_PARASITES */
   
   
@@ -846,12 +840,9 @@ load_image (gchar        *filename,
    */
   /* temporary buffer */
   tile_height = gimp_tile_height ();
-  buf = g_new (guchar, tile_height * cinfo.output_width * cinfo.output_components);
+  buf = g_new (guchar, 
+	       tile_height * cinfo.output_width * cinfo.output_components);
   
-  if (preview)
-    padded_buf = g_new (guchar, tile_height * cinfo.output_width *
-			(cinfo.output_components + 1));
-
   rowbuf = g_new (guchar*, tile_height);
 
   for (i = 0; i < tile_height; i++)
@@ -868,16 +859,37 @@ load_image (gchar        *filename,
       image_type = GIMP_GRAY;
       layer_type = preview ? GIMP_GRAYA_IMAGE : GIMP_GRAY_IMAGE;
       break;
+
     case 3:
       image_type = GIMP_RGB;
       layer_type = preview ? GIMP_RGBA_IMAGE : GIMP_RGB_IMAGE;
       break;
+
+    case 4:
+      if (cinfo.out_color_space == JCS_CMYK)
+	{
+	  image_type = GIMP_RGB;
+	  layer_type = GIMP_RGB_IMAGE;
+	  break;
+	}
+      /*fallthrough*/
+
     default:
-      g_message ("don't know how to load JPEGs\nwith %d color channels",
-                   cinfo.output_components);
+      g_message ("don't know how to load JPEGs\nwith %d color channels\nusing colorspace %d (%d)",
+		 cinfo.output_components, cinfo.out_color_space,
+		 cinfo.jpeg_color_space);
       gimp_quit ();
+      break;
     }
 
+  if (preview)
+    padded_buf = g_new (guchar, tile_height * cinfo.output_width *
+			(cinfo.output_components + 1));
+  else if (cinfo.out_color_space == JCS_CMYK)
+    padded_buf = g_new (guchar, tile_height * cinfo.output_width * 3);
+  else
+    padded_buf = NULL;
+
   if (preview) 
     {
       image_ID = image_ID_global;
@@ -891,10 +903,11 @@ load_image (gchar        *filename,
 
   if (preview) 
     {
-      layer_ID_global = layer_ID = gimp_layer_new (image_ID, _("JPEG preview"),
-						   cinfo.output_width,
-						   cinfo.output_height,
-						   layer_type, 100, GIMP_NORMAL_MODE);
+      layer_ID_global = layer_ID = 
+	gimp_layer_new (image_ID, _("JPEG preview"),
+			cinfo.output_width,
+			cinfo.output_height,
+			layer_type, 100, GIMP_NORMAL_MODE);
     }
   else 
     {
@@ -957,35 +970,31 @@ load_image (gchar        *filename,
   while (cinfo.output_scanline < cinfo.output_height)
     {
       start = cinfo.output_scanline;
-      end = cinfo.output_scanline + tile_height;
-      end = MIN (end, cinfo.output_height);
+      end   = cinfo.output_scanline + tile_height;
+      end   = MIN (end, cinfo.output_height);
       scanlines = end - start;
 
       for (i = 0; i < scanlines; i++)
 	jpeg_read_scanlines (&cinfo, (JSAMPARRAY) &rowbuf[i], 1);
 
-      /*
-      for (i = start; i < end; i++)
-	gimp_pixel_rgn_set_row (&pixel_rgn, tilerow[i - start], 0, i, drawable->width);
-	*/
-
       if (preview) /* Add a dummy alpha channel -- convert buf to padded_buf */
 	{
 	  guchar *dest = padded_buf;
 	  guchar *src  = buf;
-	  gint num = drawable->width * scanlines;
+	  gint    num  = drawable->width * scanlines;
 
 	  switch (cinfo.output_components)
 	    {
 	    case 1:
-	      for (i=0; i<num; i++)
+	      for (i = 0; i < num; i++)
 		{
 		  *(dest++) = *(src++);
 		  *(dest++) = 255;
 		}
 	      break;
+
 	    case 3:
-	      for (i=0; i<num; i++)
+	      for (i = 0; i < num; i++)
 		{
 		  *(dest++) = *(src++);
 		  *(dest++) = *(src++);
@@ -993,17 +1002,39 @@ load_image (gchar        *filename,
 		  *(dest++) = 255;
 		}
 	      break;
+
 	    default:
-	      g_warning ("JPEG - shouldn't have gotten here.  Report to adam@gimp.org");
+	      g_warning ("JPEG - shouldn't have gotten here.\nReport to http://bugzilla.gnome.org/");
+	      break;
+	    }
+	}
+      else if (cinfo.out_color_space == JCS_CMYK) /* buf-> RGB in padded_buf */
+	{
+	  guchar *dest = padded_buf;
+	  guchar *src  = buf;
+	  gint    num  = drawable->width * scanlines;
+
+	  for (i = 0; i < num; i++)
+	    {
+	      guint r_c, g_m, b_y, a_k;
+
+	      r_c = *(src++);
+	      g_m = *(src++);
+	      b_y = *(src++);
+	      a_k = *(src++);
+	      *(dest++) = (r_c * a_k) / 255;
+	      *(dest++) = (g_m * a_k) / 255;
+	      *(dest++) = (b_y * a_k) / 255;
 	    }
 	}
 
-      gimp_pixel_rgn_set_rect (&pixel_rgn, preview ? padded_buf : buf,
+      gimp_pixel_rgn_set_rect (&pixel_rgn, padded_buf ? padded_buf : buf,
 			       0, start, drawable->width, scanlines);
 
       if (runmode != GIMP_RUN_NONINTERACTIVE)
 	{
-	  gimp_progress_update ((double) cinfo.output_scanline / (double) cinfo.output_height);
+	  gimp_progress_update ((gdouble) cinfo.output_scanline / 
+				(gdouble) cinfo.output_height);
 	}
     }
 
@@ -1022,8 +1053,7 @@ load_image (gchar        *filename,
   /* free up the temporary buffers */
   g_free (rowbuf);
   g_free (buf);
-  if (preview)
-    g_free (padded_buf);
+  g_free (padded_buf);
 
   /* After finish_decompress, we can close the input file.
    * Here we postpone it until after no more JPEG errors are possible,
@@ -1052,6 +1082,7 @@ load_image (gchar        *filename,
 	  gimp_image_parasite_attach (image_ID, comment_parasite);
 	  gimp_parasite_free (comment_parasite);
 	}
+
       if (vals_parasite)
 	{
 	  gimp_image_parasite_attach (image_ID, vals_parasite);
@@ -1071,24 +1102,21 @@ load_image (gchar        *filename,
 static void
 background_error_exit (j_common_ptr cinfo)
 {
-  if (abort_me) *abort_me = 1;
+  if (abort_me) 
+    *abort_me = TRUE;
   (*cinfo->err->output_message) (cinfo);
 }
 
-static gint
-background_jpeg_save (gpointer *ptr)
+static gboolean
+background_jpeg_save (PreviewPersistent *pp)
 {
-  preview_persistent *pp = (preview_persistent *)ptr;
   guchar *t;
   guchar *s;
-  gint i, j;
-  gint yend;
+  gint    i, j;
+  gint    yend;
 
   if (pp->abort_me || (pp->cinfo.next_scanline >= pp->cinfo.image_height))
     {
-      struct stat buf;
-      gchar temp[256];
-
       /* clean up... */
       if (pp->abort_me)
 	{
@@ -1105,11 +1133,15 @@ background_jpeg_save (gpointer *ptr)
       g_free (pp->temp);
       g_free (pp->data);
 
-      if (pp->drawable) gimp_drawable_detach (pp->drawable);
+      if (pp->drawable) 
+	gimp_drawable_detach (pp->drawable);
 
       /* display the preview stuff */
       if (!pp->abort_me) 
 	{
+	  struct stat buf;
+	  gchar       temp[128];
+
 	  stat (pp->file_name, &buf);
 	  g_snprintf (temp, sizeof (temp),
 		      _("Size: %lu bytes (%02.01f kB)"),
@@ -1124,7 +1156,8 @@ background_jpeg_save (gpointer *ptr)
       unlink (pp->file_name);
       g_free (pp->file_name);
 
-      if (abort_me == &(pp->abort_me)) abort_me = NULL;
+      if (abort_me == &(pp->abort_me)) 
+	abort_me = NULL;
 
       g_free (pp);
 
@@ -1165,26 +1198,26 @@ background_jpeg_save (gpointer *ptr)
     }
 }
 
-static gint
-save_image (char   *filename,
-	    gint32  image_ID,
-	    gint32  drawable_ID,
-	    gint32  orig_image_ID,
-	    int     preview)
+static gboolean
+save_image (gchar    *filename,
+	    gint32    image_ID,
+	    gint32    drawable_ID,
+	    gint32    orig_image_ID,
+	    gboolean  preview)
 {
-  GimpPixelRgn pixel_rgn;
-  GimpDrawable *drawable;
-  GimpImageType drawable_type;
+  GimpPixelRgn   pixel_rgn;
+  GimpDrawable  *drawable;
+  GimpImageType  drawable_type;
   struct jpeg_compress_struct cinfo;
-  struct my_error_mgr jerr;
-  FILE * volatile outfile;
-  guchar *temp, *t;
-  guchar *data;
-  guchar *src, *s;
-  gchar *name;
-  gint has_alpha;
-  gint rowstride, yend;
-  gint i, j;
+  struct my_error_mgr         jerr;
+  FILE     * volatile outfile;
+  guchar   *temp, *t;
+  guchar   *data;
+  guchar   *src, *s;
+  gchar    *name;
+  gboolean  has_alpha;
+  gint      rowstride, yend;
+  gint      i, j;
 
   drawable = gimp_drawable_get (drawable_ID);
   drawable_type = gimp_drawable_type (drawable_ID);
@@ -1250,19 +1283,22 @@ save_image (char   *filename,
     case GIMP_GRAY_IMAGE:
       /* # of color components per pixel */
       cinfo.input_components = drawable->bpp;
-      has_alpha = 0;
+      has_alpha = FALSE;
       break;
+
     case GIMP_RGBA_IMAGE:
     case GIMP_GRAYA_IMAGE:
       /*gimp_message ("jpeg: image contains a-channel info which will be lost");*/
       /* # of color components per pixel (minus the GIMP alpha channel) */
       cinfo.input_components = drawable->bpp - 1;
-      has_alpha = 1;
+      has_alpha = TRUE;
       break;
+
     case GIMP_INDEXED_IMAGE:
       /*gimp_message ("jpeg: cannot operate on indexed color images");*/
       return FALSE;
       break;
+
     default:
       /*gimp_message ("jpeg: cannot operate on unknown image types");*/
       return FALSE;
@@ -1275,7 +1311,7 @@ save_image (char   *filename,
    * Four fields of the cinfo struct must be filled in:
    */
   /* image width and height, in pixels */
-  cinfo.image_width = drawable->width;
+  cinfo.image_width  = drawable->width;
   cinfo.image_height = drawable->height;
   /* colorspace of input image */
   cinfo.in_color_space = (drawable_type == GIMP_RGB_IMAGE ||
@@ -1287,8 +1323,8 @@ save_image (char   *filename,
    */
   jpeg_set_defaults (&cinfo);
 
-  jpeg_set_quality (&cinfo, (int) (jsvals.quality * 100), jsvals.baseline);
-  cinfo.smoothing_factor = (int) (jsvals.smoothing * 100);
+  jpeg_set_quality (&cinfo, (gint) (jsvals.quality * 100), jsvals.baseline);
+  cinfo.smoothing_factor = (gint) (jsvals.smoothing * 100);
   cinfo.optimize_coding = jsvals.optimize;
 
 #ifdef HAVE_PROGRESSIVE_JPEG
@@ -1309,6 +1345,7 @@ save_image (char   *filename,
       cinfo.comp_info[2].h_samp_factor = 1;
       cinfo.comp_info[2].v_samp_factor = 1;
       break;
+
     case 1:
       cinfo.comp_info[0].h_samp_factor = 2;
       cinfo.comp_info[0].v_samp_factor = 1;
@@ -1317,6 +1354,7 @@ save_image (char   *filename,
       cinfo.comp_info[2].h_samp_factor = 1;
       cinfo.comp_info[2].v_samp_factor = 1;
       break;
+
     case 2:
       cinfo.comp_info[0].h_samp_factor = 1;
       cinfo.comp_info[0].v_samp_factor = 1;
@@ -1336,9 +1374,11 @@ save_image (char   *filename,
     default:
       cinfo.dct_method = JDCT_ISLOW;
       break;
+
     case 1:
       cinfo.dct_method = JDCT_IFAST;
       break;
+
     case 2:
       cinfo.dct_method = JDCT_FLOAT;
       break;
@@ -1402,8 +1442,8 @@ save_image (char   *filename,
    */
   /* JSAMPLEs per row in image_buffer */
   rowstride = drawable->bpp * drawable->width;
-  temp = (guchar *) g_malloc (cinfo.image_width * cinfo.input_components);
-  data = (guchar *) g_malloc (rowstride * gimp_tile_height ());
+  temp = g_new (guchar, cinfo.image_width * cinfo.input_components);
+  data = g_new (guchar, rowstride * gimp_tile_height ());
 
   /* fault if cinfo.next_scanline isn't initially a multiple of
    * gimp_tile_height */
@@ -1416,29 +1456,28 @@ save_image (char   *filename,
 
   if (preview) 
     {
-      preview_persistent *pp = g_malloc (sizeof (preview_persistent));
-      if (pp == NULL) return FALSE;
+      PreviewPersistent *pp = g_new (PreviewPersistent, 1);
       
       /* pass all the information we need */
-      pp->cinfo = cinfo;
+      pp->cinfo       = cinfo;
       pp->tile_height = gimp_tile_height();
-      pp->data = data;
-      pp->outfile = outfile;
-      pp->has_alpha = has_alpha;
-      pp->rowstride = rowstride;
-      pp->temp = temp;
-      pp->data = data;
-      pp->drawable = drawable;
-      pp->pixel_rgn = pixel_rgn;
-      pp->src = NULL;
-      pp->file_name = filename;
+      pp->data        = data;
+      pp->outfile     = outfile;
+      pp->has_alpha   = has_alpha;
+      pp->rowstride   = rowstride;
+      pp->temp        = temp;
+      pp->data        = data;
+      pp->drawable    = drawable;
+      pp->pixel_rgn   = pixel_rgn;
+      pp->src         = NULL;
+      pp->file_name   = filename;
       
-      pp->abort_me = 0;
+      pp->abort_me    = FALSE;
       abort_me = &(pp->abort_me);
       
       jerr.pub.error_exit = background_error_exit;
  
-      gtk_idle_add ((GtkFunction) background_jpeg_save, (gpointer *)pp);
+      gtk_idle_add ((GtkFunction) background_jpeg_save, (gpointer) pp);
       
       /* background_jpeg_save() will cleanup as needed */
       return TRUE;
@@ -1450,7 +1489,9 @@ save_image (char   *filename,
 	{
 	  yend = cinfo.next_scanline + gimp_tile_height ();
 	  yend = MIN (yend, cinfo.image_height);
-	  gimp_pixel_rgn_get_rect (&pixel_rgn, data, 0, cinfo.next_scanline, cinfo.image_width,
+	  gimp_pixel_rgn_get_rect (&pixel_rgn, data, 
+				   0, cinfo.next_scanline, 
+				   cinfo.image_width,
 				   (yend - cinfo.next_scanline));
 	  src = data;
 	}
@@ -1471,7 +1512,8 @@ save_image (char   *filename,
       jpeg_write_scanlines (&cinfo, (JSAMPARRAY) &temp, 1);
 
       if ((cinfo.next_scanline % 5) == 0)
-	gimp_progress_update ((double) cinfo.next_scanline / (double) cinfo.image_height);
+	gimp_progress_update ((gdouble) cinfo.next_scanline / 
+			      (gdouble) cinfo.image_height);
     }
 
   /* Step 6: Finish compression */
@@ -1504,9 +1546,12 @@ make_preview (void)
 
   if (jsvals.preview) 
     {
-      tn = tempnam(NULL, "gimp");	/* user temp dir? */
-      save_image (tn, image_ID_global,
-		  drawable_ID_global, orig_image_ID_global, TRUE);
+      tn = gimp_temp_name ("jpeg");
+      save_image (tn, 
+		  image_ID_global,
+		  drawable_ID_global, 
+		  orig_image_ID_global, 
+		  TRUE);
     }
   else 
     {
@@ -1516,7 +1561,6 @@ make_preview (void)
       gimp_displays_flush ();
       gdk_flush();
     }
-
 }
 
 static void
@@ -1524,37 +1568,27 @@ destroy_preview (void)
 {
   if (abort_me) 
     {
-      *abort_me = 1;   /* signal the background save to stop */
+      *abort_me = TRUE;   /* signal the background save to stop */
     }
+
   if (drawable_global) 
     {
       gimp_drawable_detach (drawable_global);
       drawable_global = NULL;
     }
+
   if (layer_ID_global != -1 && image_ID_global != -1) 
     {
+      /*  assuming that reference counting is working correctly, 
+	  we do not need to delete the layer, removing it from
+	  the image should be sufficient  */
       gimp_image_remove_layer (image_ID_global, layer_ID_global);
       
-      /* gimp_layer_delete(layer_ID_global); */
       layer_ID_global = -1;
     }
 }
 
-static void
-add_menu_item (GtkWidget        *menu, 
-	       gchar            *label, 
-	       guint             op_no, 
-	       GtkSignalFunc     callback)
-{
-  GtkWidget *menu_item = gtk_menu_item_new_with_label (label);
-
-  gtk_container_add (GTK_CONTAINER (menu), menu_item);
-  gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
-		      (GtkSignalFunc) callback, &op_no);
-  gtk_widget_show (menu_item);
-}
-
-static gint
+static gboolean
 save_dialog (void)
 {
   GtkWidget *dlg;
@@ -1569,15 +1603,13 @@ save_dialog (void)
   GtkObject *scale_data;
 
   GtkWidget *progressive;
-  GtkWidget *baseline;
+  /*   GtkWidget *baseline;  */
   GtkWidget *restart;
 
   GtkWidget *preview;
   /* GtkWidget *preview_size; -- global */
 
   GtkWidget *menu;
-  GtkWidget *subsmp_menu;
-  GtkWidget *dct_menu;
   
   GtkWidget *text;
   GtkWidget *com_frame;
@@ -1644,7 +1676,7 @@ save_dialog (void)
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
   gtk_box_pack_start (GTK_BOX (main_vbox), frame, FALSE, FALSE, 0);
 
-  table = gtk_table_new (9, 3, FALSE);
+  table = gtk_table_new (8, 3, FALSE);
   gtk_table_set_col_spacings (GTK_TABLE (table), 4);
   gtk_table_set_row_spacings (GTK_TABLE (table), 4);
   gtk_container_set_border_width (GTK_CONTAINER (table), 4);
@@ -1765,64 +1797,58 @@ save_dialog (void)
   gtk_widget_set_sensitive (progressive, FALSE);
 #endif
   
-  baseline = gtk_check_button_new_with_label (_("Force baseline JPEG (Readable by all decoders)"));
-  gtk_table_attach (GTK_TABLE (table), baseline, 0, 3, 6, 7,
-		    GTK_FILL, 0, 0, 0);
-  gtk_signal_connect (GTK_OBJECT (baseline), "toggled",
-		      GTK_SIGNAL_FUNC (gimp_toggle_button_update),
-		      &jsvals.baseline);
-  gtk_signal_connect (GTK_OBJECT (baseline), "toggled",
-		      GTK_SIGNAL_FUNC (make_preview),
-		      NULL);
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (baseline),
-				jsvals.baseline);
-  gtk_widget_show (baseline);
+  /* 
+   * Commented out since it triggers a bug in libjpeg if
+   * used with low quality setting (bug #57727).
+   *
+   * baseline = gtk_check_button_new_with_label (_("Force baseline JPEG (Readable by all decoders)"));
+   * gtk_table_attach (GTK_TABLE (table), baseline, 0, 3, 6, 7,
+   * 		    GTK_FILL, 0, 0, 0);
+   * gtk_signal_connect (GTK_OBJECT (baseline), "toggled",
+   * 		      GTK_SIGNAL_FUNC (gimp_toggle_button_update),
+   * 		      &jsvals.baseline);
+   * gtk_signal_connect (GTK_OBJECT (baseline), "toggled",
+   *		      GTK_SIGNAL_FUNC (make_preview),
+   *		      NULL);
+   * gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (baseline),
+   *				jsvals.baseline);
+   * gtk_widget_show (baseline);
+   */
 
-  /* build option menu (code taken from app/buildmenu.c) */
-  menu = gtk_menu_new ();
+  /* Subsampling */
+  menu = 
+    gimp_option_menu_new2 (FALSE, 
+			   menu_callback, 
+			   &jsvals.subsmp, GINT_TO_POINTER (jsvals.subsmp),
+			   "2x2,1x1,1x1",  GINT_TO_POINTER (0), NULL, 
+			   "2x1,1x1,1x1 (4:2:2)", GINT_TO_POINTER (1), NULL,
+			   "1x1,1x1,1x1",  GINT_TO_POINTER (2), NULL,
+			   NULL);
 
-  add_menu_item (menu, "2x2,1x1,1x1", 0, subsmp_callback);
-  add_menu_item (menu, "2x1,1x1,1x1 (4:2:2)", 1, subsmp_callback);
-  add_menu_item (menu, "1x1,1x1,1x1", 2, subsmp_callback);
-
-  subsmp_menu = gtk_option_menu_new ();
-
-  label = gtk_label_new (_("Subsampling:"));
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-  gtk_table_attach (GTK_TABLE (table), label, 0, 2, 7, 8,
-		    GTK_FILL | GTK_SHRINK, GTK_FILL, 0, 0);
-  gtk_table_attach (GTK_TABLE (table), subsmp_menu, 2, 3, 7, 8,
-		    GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
-  gtk_widget_show (label);
-  gtk_widget_show (subsmp_menu);
-  gtk_option_menu_set_menu (GTK_OPTION_MENU (subsmp_menu), menu);
+  gimp_table_attach_aligned (GTK_TABLE (table), 1, 6, 
+			     _("Subsampling:"),
+			     1.0, 0.5,
+			     menu, 1, FALSE);
 
   /* DCT method */
-  menu = gtk_menu_new ();
+  menu = 
+    gimp_option_menu_new2 (FALSE, 
+			   menu_callback, 
+			   &jsvals.dct, GINT_TO_POINTER (jsvals.dct),
+			   _("Fast Integer"),   GINT_TO_POINTER (1), NULL, 
+			   _("Integer"),        GINT_TO_POINTER (0), NULL,
+			   _("Floating-Point"), GINT_TO_POINTER (2), NULL,
+			   NULL);
 
-  add_menu_item (menu, _("Fast integer"),   1, dct_callback);
-  add_menu_item (menu, _("Integer"),        0, dct_callback);
-  add_menu_item (menu, _("Floating-point"), 2, dct_callback);
-
-  dct_menu = gtk_option_menu_new ();
- 
-  label = gtk_label_new (_("DCT method (Speed/quality tradeoff):"));
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-  gtk_table_attach (GTK_TABLE (table), label, 0, 2, 8, 9,
-		    GTK_FILL | GTK_SHRINK, GTK_FILL, 0, 0);
-  gtk_table_attach (GTK_TABLE (table), dct_menu, 2, 3, 8, 9,
-		    GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
-  gtk_widget_show (label);
-  gtk_widget_show (dct_menu);
-  gtk_option_menu_set_menu (GTK_OPTION_MENU (dct_menu), menu);
-  gtk_option_menu_set_history (GTK_OPTION_MENU (dct_menu), 1);
+  gimp_table_attach_aligned (GTK_TABLE (table), 1, 7, 
+			     _("DCT method (Speed/quality tradeoff):"),
+			     1.0, 0.5,
+			     menu, 1, FALSE);
 
   dtype = gimp_drawable_type (drawable_ID_global);
   if (dtype != GIMP_RGB_IMAGE && dtype != GIMP_RGBA_IMAGE) 
-    {
-      gtk_widget_set_sensitive (subsmp_menu, FALSE);
-    }
-
+    gtk_widget_set_sensitive (menu, FALSE);
+ 
   com_frame = gtk_frame_new (_("Image comments"));
   gtk_frame_set_shadow_type (GTK_FRAME (com_frame), GTK_SHADOW_ETCHED_IN);
   gtk_box_pack_start (GTK_BOX (main_vbox), com_frame, TRUE, TRUE, 0);
@@ -1886,21 +1912,21 @@ save_ok_callback (GtkWidget *widget,
 		  gpointer   data)
 {
   GtkWidget *text;
+
   jsint.run = TRUE;
 
   /* pw - get the comment text object and grab it's data */
   text = gtk_object_get_data (GTK_OBJECT (data), "text");
-  if (image_comment != NULL) 
-    {
-      g_free (image_comment);
-      image_comment = NULL;
-    }
   
   /* pw - gtk_editable_get_chars allocates a copy of the string, so
    * don't worry about the gtk_widget_destroy killing it.  */
 
-  image_comment = gtk_editable_get_chars (GTK_EDITABLE (text), 0, -1);
-  
+  g_free (image_comment);
+  image_comment = NULL;
+
+  if (text)
+    image_comment = gtk_editable_get_chars (GTK_EDITABLE (text), 0, -1);
+
   gtk_widget_destroy (GTK_WIDGET (data));
 }
 
@@ -1926,17 +1952,9 @@ save_restart_update (GtkAdjustment *adjustment,
 }
 
 static void
-subsmp_callback (GtkWidget *widget,
-		 gpointer   data)
+menu_callback (GtkWidget *widget,
+	       gpointer   data)
 {
-  jsvals.subsmp = *((guchar *) data);
-  make_preview ();
-}
-
-static void
-dct_callback (GtkWidget *widget,
-	      gpointer   data)
-{
-  jsvals.dct = *((guchar *) data);
+  gimp_menu_item_update (widget, data);
   make_preview ();
 }
